/*
 ********************************************************
	Rug Warrior Pro UMH-ISA (SITR)
 ********************************************************
	NOMBRE DEL ARCHIVO:   	servovel.c
	FECHA:			
	AUTOR:   Luis M. Jimnez
	DESCRIPCIN:  Implementa un servo de velocidad
			realimentando la velocidad medida por los encoders
			Requiere: defines.c, comun.c
				  
	VARIABLES GLOBALES EXTERNAS:
		float control_interval = 0.250;	
		float k_integral = 0.10;		
		float k_pro =1.0;				

    FUNCIONES EXTERNAS: 
		void start_speed_control(int vel_tras, int vel_rot);
		void stop_speed_control();
		void drivev(int vel_tras, int vel_rot);

	VARIABLES GLOBALES USO INTERNO:
		float CLICKS_PER_TURN= 8.0 / 100.0; 
		float des_vel_clicks = 0.0;		
		float des_bias_clicks = 0.0;		
		float integral =0.0;			
		float power[2] = {0, 0};		
		int pid_speed;					

	FUNCIONES USO INTERNO: 
		void alter_power(float left_error, float right_error);
		float integrate(float left_vel, float right_vel, float bias);
		void speed_control();

  ********************************************************
*/

float CLICKS_PER_TURN= 8.0 / 100.0; /* clicks por vuelta */

int control_interval = 250;	/* Periodo del bucle de control ms */
float des_vel_clicks = 0.0;		/* Velocidad deseada */
float des_bias_clicks = 0.0;		/* Bias entre motores deseada (cliks/seg) */
float integral =0.0;			/* Integral de la diferencia de velocidades */
float k_integral = 0.10;		/* Contante de integracin */
float k_pro =1.0;				/* Ganancia del regulador proporcional */
float power[2] = {0.0, 0.0};			/* Potencia enviada a los motores */
int pid_speed;					/* identificador proceso */

void alter_power(float left_error, float right_error)
{
	power[MOTOR_LEFT]= flimit(power[MOTOR_LEFT]+left_error, -100.0, 100.0);
	power[MOTOR_RIGHT]= flimit(power[MOTOR_RIGHT]+right_error, -100.0, 100.0);

	motor(MOTOR_LEFT, (int)power[MOTOR_LEFT]);
	motor(MOTOR_RIGHT, (int)power[MOTOR_RIGHT]);
}

/* calcula error de velocidad */
float integrate(float left_vel, float right_vel, float bias)
{
	integral = flimit(integral + left_vel + bias - right_vel,
				-1000.0, 1000.0);
	return integral;
}

/* inicializa  el servo de velocidad para una velocidad
de traslacin y rotacin [-100, 100] */
void start_speed_control(int vel_tras, int vel_rot)
{
	des_vel_clicks= CLICKS_PER_TURN * (float)vel_tras;
	des_bias_clicks= CLICKS_PER_TURN * (float)vel_rot;
	
	init_velocity();
	get_left_clicks();	/* reset counters */
	get_right_clicks();

	integral=0.0;	/* init variables */
	power[MOTOR_LEFT]= 0.0;
	power[MOTOR_RIGHT]= 0.0;
	stop();	/* por si el robot ya estaba en movimiento */

	pid_speed=start_process(speed_control());
}


/* Modifica los parmetros de velocidad del servo:
velocidad de  traslacin y rotacin [-100, 100] 
debe ejecutarse de forma atmica (p.e. como un proceso)*/
void drivev(int vel_tras, int vel_rot)
{
	des_vel_clicks= CLICKS_PER_TURN * (float)vel_tras;
	des_bias_clicks= CLICKS_PER_TURN * (float)vel_rot;
	integral=0.0;	/* init variables */

}

void stop_speed_control()
{
	kill_process(pid_speed);

	/* stop moving */
	stop();
}

void speed_control()
{
	float left_vel, right_vel, bias;
	float integral_error, left_error, right_error;
	while(1)
	{
		/* obtiene velocidad actual */
		left_vel = (float)get_left_clicks();
		right_vel = (float)get_right_clicks();

	   if((des_vel_clicks==0.0) && (des_bias_clicks==0.0)) /* servo off */
	   {
		stop();
		power[MOTOR_LEFT]= 0.0;
		power[MOTOR_RIGHT]= 0.0;
		integral=0.0;
	   }
	   else
	   {
		integral_error = k_integral *
			integrate (left_vel, right_vel, des_bias_clicks);
		left_error = k_pro *
			(des_vel_clicks - left_vel -integral_error);
		right_error = k_pro *
			(des_vel_clicks - right_vel +integral_error);
		alter_power(left_error, right_error);		
   	    }
	    wait(control_interval);

	}
}
